/// <reference types="angular" />
import { CodeMap } from "./model/CodeMap";
import { IRootScopeService, IAngularEvent } from "angular";
import { DeltaCalculatorService } from "./data.deltaCalculator.service";
import { DataDecoratorService } from "./data.decorator.service";
export interface DataModel {
    revisions: CodeMap[];
    metrics: string[];
    renderMap: CodeMap;
}
export interface DataServiceSubscriber {
    onDataChanged(data: DataModel, event: IAngularEvent): any;
}
/**
 * This service stores and sets the current revisions, map and metrics
 */
export declare class DataService {
    private $rootScope;
    private deltaCalculatorService;
    private dataDecoratorService;
    private _data;
    private _lastReferenceIndex;
    private _lastComparisonMap;
    private _deltasEnabled;
    constructor($rootScope: IRootScopeService, deltaCalculatorService: DeltaCalculatorService, dataDecoratorService: DataDecoratorService);
    setMap(map: CodeMap, revision: number): void;
    setReferenceMap(index: number): void;
    setComparisonMap(index: number): void;
    private processDeltas();
    onActivateDeltas(): void;
    onDeactivateDeltas(): void;
    readonly data: DataModel;
    subscribe(subscriber: DataServiceSubscriber): void;
    notify(): void;
    getReferenceMapName(): string;
    getComparisonMapName(): string;
    getReferenceMap(): CodeMap;
    getComparisonMap(): CodeMap;
    updateMetrics(): void;
    /**
     * resets all maps (deletes them)
     */
    resetMaps(): void;
}
